'''''''''''''''''''''''''''''''''''''''''''
' Program    : Loads TGA Files            '
' Name       : OPENTGA Version 0.1        '
' Programmer : Yousuf Philips             '
' Company    : Y P I                      '
' Updated On : 12th of June 1999          '
' Email      : philipz85@hotmail.com      '
' Website    : http://www.philipz.com/qb/ '
'''''''''''''''''''''''''''''''''''''''''''

'/*               Do not edit this file if you distribute it.              */'
'/*  (c) Copyrighted by YPI in 1998 | All Rights Reserved | Public Domain  */'

'/* This program has been created by YPI (Basic Programming Incorporation) */'
'/* as a utility to view True Vision Targa Image files [IFF/LBM].          */'
'/* This program currently loads 8, 16, 24-bit TGA files.                  */'

'/* I would like to give thanks to Max Maischein [File Format List]        */'

'/* If you use any of this code in your program then you must credit YPI.  */'
'/* YPI has created an assortment of Image viewers and players and they    */'
'/* can all be found at <http://www.philipz.com/qb/>                       */'
'/* Please send comments & suggestions to philipz85@hotmail.com            */'

DECLARE SUB OpenTGA (FileName$)

TYPE TGAHeader
   LengthOfField AS STRING * 1    ' Length of the Image Identification Field
   ColorMap AS STRING * 1         ' Color Map Type - 0 [No Map] 1 [256 Entry]
   ImageType AS STRING * 1        ' Image Type
                                  ' [0] No Image Data Included
                                  ' [1] Uncompressed Color Map Image [4,8]
                                  ' [2] Uncompressed RGB Image [16,24]
                                  ' [3] Uncompressed Black & White
                                  ' [9] RLE Color Map Image
                                  ' [10] RLE RGB Image
                                  ' [11] RLE Black & White
                                  ' [32 | 33] Compressed Color Map using
                                  '           Huffman, Delta, RLE.
   OffsetOfColorMap AS INTEGER    ' Offset of first color map entry (Not Imp)
   ColorsUsed AS INTEGER          ' Number of color map entries (Not Imp)
   BitsPerColorMap AS STRING * 1  ' Number of bits per color map entry(Not Imp)
   XCoordinate AS INTEGER         ' Image X Co-ordinate of lower left corner(Not Imp)
   YCoordinate AS INTEGER         ' Image Y Co-ordinate of lower left corner(Not Imp)
   ImageWidth AS INTEGER          ' Width of image in pixels
   ImageHeight AS INTEGER         ' Height of image in pixels
   BitsPerPixel AS STRING * 1     ' Bits per pixel
   Flags AS STRING * 1            ' Flags (Not Imp)
   '/* Following the header is the Image Identification Field. Extract the
   '   number of bytes specified in LengthOfField
   '/* Following this is the Color Map Data
   '   If BitsPerColorMap = 32 then 1 byte for Red, Green, Blue & Attribute
   '   If BitsPerColorMap = 24 then 1 byte for Red, Green & Blue
   '   If BitsPerColorMap = 16 then 2 bytes [ARRRRRGG GGGBBBBB]
   '/* Following the color map is the image data
END TYPE

DEFINT A-Z
ON ERROR GOTO Errrr

IF COMMAND$ <> "" THEN
FileName$ = COMMAND$
IF RIGHT$(UCASE$(FileName$), 4) <> ".TGA" THEN
   FileName$ = FileName$ + ".TGA"
END IF
CALL OpenTGA(FileName$)
SYSTEM
ELSE
  PRINT "OpenTGA v0.1"
    PRINT "(C) Data Components Software Development"
      PRINT "Code by: Yousuf Philips of Y P I, Inc."
        PRINT
          PRINT "Usage: OPENTGA[.exe] TGAFILE.TGA"
            SYSTEM
END IF
Errrr:
RESUME NEXT

SUB OpenTGA (FileName$)

DIM TGAHeader AS TGAHeader

CLS
OPEN FileName$ FOR BINARY AS #1
IF LOF(1) = 0 THEN
   PRINT " File Is Empty"
   CLOSE : KILL FileName$: EXIT SUB
END IF

GET #1, , TGAHeader
PRINT
IF ASC(TGAHeader.LengthOfField) <> 0 THEN
   PRINT " Length of Image Identification Field "; ASC(TGAHeader.LengthOfField)
   ImageIdentificationField$ = SPACE$(ASC(TGAHeader.LengthOfField))
END IF
IF ASC(TGAHeader.ColorMap) = 0 THEN
   PRINT " TGA Has No Color Map"
ELSEIF ASC(TGAHeader.ColorMap) = 1 THEN
   PRINT " TGA Has 256 Color Map"
ELSE
   PRINT " Not A Valid TGA File"
   EXIT SUB
END IF
Movement = ASC(TGAHeader.BitsPerColorMap) \ 8
BitsPerPixel = ASC(TGAHeader.BitsPerPixel)
ImageType = ASC(TGAHeader.ImageType)

SELECT CASE ImageType
   CASE 0: PRINT " No Image Data Available"
   CASE 1: PRINT " Uncompressed Color Image"
           Palette$ = SPACE$(Movement * TGAHeader.ColorsUsed)
   CASE 2: PRINT " Uncompressed RGB Image"
   CASE 3: PRINT " Uncompressed Black & White Image"
   CASE 9: PRINT " Compressed Color Image"
           Palette$ = SPACE$(Movement * TGAHeader.ColorsUsed)
   CASE 10: PRINT " Compressed RGB Image"
   CASE 11: PRINT " Compressed Black & White Image"
END SELECT

PRINT " Width Of Image -"; TGAHeader.ImageWidth
PRINT " Height Of Image -"; TGAHeader.ImageHeight
PRINT " Bits Per Pixel -"; BitsPerPixel
GET #1, , ImageIdentificationField$
PRINT ImageIdentificationField$

IF ImageType = 3 OR ImageType = 11 THEN
   PRINT " Can't Display Image | Image Type Not Openable"
   EXIT SUB
ELSEIF ASC(TGAHeader.BitsPerColorMap) = 16 THEN
   PRINT " Can't Display Image | ColorMap - To Small"
   EXIT SUB
ELSEIF ASC(TGAHeader.BitsPerPixel) <> 8 AND ASC(TGAHeader.BitsPerPixel) <> 16 AND ASC(TGAHeader.BitsPerPixel) <> 24 THEN
   PRINT " Can't Display Image | BitsPerPixel -"; ASC(TGAHeader.BitsPerPixel)
   EXIT SUB
END IF
PRINT " Press A Key To Display Image"
Wait$ = INPUT$(1)

SCREEN 13
IF Palette$ <> "" THEN
   GET #1, , Palette$
   OUT &H3C8, 0
   FOR Loops = 0 TO (2 ^ BitsPerPixel) - 1
      Blue = ASC(MID$(Palette$, Loops * Movement + 1, 1)) \ 4
      Green = ASC(MID$(Palette$, Loops * Movement + 2, 1)) \ 4
      Red = ASC(MID$(Palette$, Loops * Movement + 3, 1)) \ 4
      OUT &H3C9, Red
      OUT &H3C9, Green
      OUT &H3C9, Blue
   NEXT Loops
ELSE
   OUT &H3C8, 0
   FOR Loops = 0 TO 255
      OUT &H3C9, Loops \ 4
      OUT &H3C9, Loops \ 4
      OUT &H3C9, Loops \ 4
   NEXT Loops
END IF

IF ImageType = 1 OR ImageType = 2 THEN
      Extract$ = SPACE$(TGAHeader.ImageWidth * (BitsPerPixel \ 8))
      FOR YHeight = TGAHeader.ImageHeight - 1 TO 0 STEP -1
         GET #1, , Extract$
         IF BitsPerPixel = 8 THEN
            FOR XWidth = 0 TO TGAHeader.ImageWidth - 1
                PSET (XWidth, YHeight), ASC(MID$(Extract$, XWidth + 1, 1))
            NEXT XWidth
         ELSEIF BitsPerPixel = 16 THEN
            FOR XWidth = 0 TO TGAHeader.ImageWidth - 1
               Bit16 = CVI(MID$(Extract$, (XWidth * 2) + 1, 2))
               Red = Bit16 AND 31: Bit16 = Bit16 \ 32
               Green = Bit16 AND 31: Bit16 = Bit16 \ 32
               Blue = Bit16 AND 31
               PSET (XWidth, YHeight), ((Red + Green + Blue) * 8) \ 3
            NEXT XWidth
         ELSEIF BitsPerPixel = 24 THEN
            FOR XWidth = 0 TO TGAHeader.ImageWidth - 1
               Red = ASC(MID$(Extract$, (XWidth * 3) + 1, 3))
               Green = ASC(MID$(Extract$, (XWidth * 3) + 2, 3))
               Blue = ASC(MID$(Extract$, (XWidth * 3) + 3, 3))
               PSET (XWidth, YHeight), (Red + Green + Blue) \ 3
            NEXT XWidth
         ELSE
            EXIT SUB
         END IF
      NEXT YHeight
ELSEIF ImageType = 9 OR ImageType = 10 THEN
      YHeight = TGAHeader.ImageHeight: Byte$ = SPACE$(1)
      BytesPerPixel = (BitsPerPixel \ 8)
      DO
         GET #1, , Byte$
         SELECT CASE ASC(Byte$)
            CASE 0 TO 127:
               ExtractBytes$ = SPACE$((ASC(Byte$) + 1) * BytesPerPixel)
               GET #1, , ExtractBytes$
               IF BitsPerPixel = 8 THEN
                  FOR Movement = 0 TO ASC(Byte$)
                     PSET (XWidth, YHeight), ASC(MID$(ExtractBytes$, Movement + 1, 1))
                     XWidth = XWidth + 1
                  NEXT Movement
               ELSEIF BitsPerPixel = 16 THEN
                  FOR Movement = 0 TO ASC(Byte$)
                     Bit16 = CVI(MID$(ExtractBytes$, (Movement * 2) + 1, 2))
                     Red = Bit16 AND 31: Bit16 = Bit16 \ 32
                     Green = Bit16 AND 31: Bit16 = Bit16 \ 32
                     Blue = Bit16 AND 31
                     PSET (XWidth, YHeight), ((Red + Green + Blue) * 8) \ 3
                     XWidth = XWidth + 1
                  NEXT Movement
               ELSEIF BitsPerPixel = 24 THEN
                  FOR Movement = 0 TO ASC(Byte$)
                     Red = ASC(MID$(ExtractBytes$, (Movement * 3) + 1, 1))
                     Green = ASC(MID$(ExtractBytes$, (Movement * 3) + 2, 1))
                     Blue = ASC(MID$(ExtractBytes$, (Movement * 3) + 3, 1))
                     PSET (XWidth, YHeight), (Red + Green + Blue) \ 3
                     XWidth = XWidth + 1
                  NEXT Movement
               END IF
            CASE 128
            CASE ELSE
               N = ASC(Byte$) AND 127
               ExtractBytes$ = SPACE$(BytesPerPixel)
               GET #1, , ExtractBytes$
               IF BitsPerPixel = 8 THEN
                  LINE (XWidth, YHeight)-STEP(N, 0), ASC(ExtractBytes$)
                  XWidth = XWidth + N + 1
               ELSEIF BitsPerPixel = 16 THEN
                  Bit16 = CVI(ExtractBytes$)
                  Red = Bit16 AND 31: Bit16 = Bit16 \ 32
                  Green = Bit16 AND 31: Bit16 = Bit16 \ 32
                  Blue = Bit16 AND 31
                  LINE (XWidth, YHeight)-STEP(N, 0), ((Red + Green + Blue) * 8) \ 3
                  XWidth = XWidth + N + 1
               ELSEIF BitsPerPixel = 24 THEN
                  Red = ASC(MID$(ExtractBytes$, 1, 1))
                  Green = ASC(MID$(ExtractBytes$, 2, 1))
                  Blue = ASC(MID$(ExtractBytes$, 3, 1))
                  LINE (XWidth, YHeight)-STEP(N, 0), (Red + Green + Blue) \ 3
                  XWidth = XWidth + N + 1
               END IF
         END SELECT
         IF XWidth = TGAHeader.ImageWidth THEN
            XWidth = 0: YHeight = YHeight - 1
         END IF
         IF YHeight = 0 THEN
            EXIT DO
         END IF
      LOOP
END IF
CLOSE
Wait$ = INPUT$(1)

END SUB

